/**
 * Filename    : Curve.hpp
 * Description : SBML Layout Curve C++ Header
 * Organization: European Media Laboratories Research gGmbH
 * Created     : 2004-07-15
 *
 * Copyright 2004 European Media Laboratories Research gGmbH
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
 * documentation provided hereunder is on an "as is" basis, and the
 * European Media Laboratories Research gGmbH have no obligations to provide maintenance, support,
 * updates, enhancements or modifications.  In no event shall the
 * European Media Laboratories Research gGmbH be liable to any party for direct, indirect, special,
 * incidental or consequential damages, including lost profits, arising
 * out of the use of this software and its documentation, even if the
 * European Media Laboratories Research gGmbH have been advised of the possibility of such damage.  See
 * the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Bioinformatics Group
 *     European Media Laboratories Research gGmbH
 *     Schloss-Wolfsbrunnenweg 31c
 *     69118 Heidelberg
 *     Germany
 *
 *     http://www.eml-research.de/english/Research/BCB/
 *     mailto:ralph.gauges@eml-r.villa-bosch.de
 *
 * Contributor(s):
 */

#ifndef Curve_HPP__
#define Curve_HPP__

#include <string>
#include "sbml/SBase.hpp"
#include "sbml/ListOf.hpp"
#include "sbml/layout/LineSegment.hpp"
#include "sbml/layout/CubicBezier.hpp"


class Curve:public SBase{
    protected:
        std::string id;
        ListOf curveSegments;

    public:

        /**
         * Creates a curve with an empty list of segments.
         */ 
        LIBSBML_EXTERN
        Curve();

        /**
         * Destructor.
         */ 
        LIBSBML_EXTERN
        virtual ~Curve();

        /**
         * Does nothing since no defaults are defined for Curve.
         */ 
        LIBSBML_EXTERN
        void initDefaults();

        /**
         * Sets the id of the curve.
         */ 
        LIBSBML_EXTERN
        void setId(const std::string& id);

        /**
         * Returns the id of the curve.
         */ 
        LIBSBML_EXTERN
        const std::string& getId() const;

        /**
         * Returns a reference to the ListOf object 
         * that holds all the curve segments.
         */
        LIBSBML_EXTERN
        const ListOf& getListOfCurveSegments() const;
       
        /**
         * Returns a refernce to the ListOf object 
         * That holds all the curve segments.
         */
        LIBSBML_EXTERN
        ListOf& getListOfCurveSegments();

        /**
         * Returns a pointer to the curve segment with the given index.
         * If the index is invalid, NULL is returned.
         */  
        LIBSBML_EXTERN
        LineSegment* getCurveSegment(unsigned int index) const;

        /**
         * Adds a new CurveSegment to the end of the list.
         */ 
        LIBSBML_EXTERN
        void addCurveSegment(LineSegment& segment);

        /**
         * Returns the number of curve segments.
         */ 
        LIBSBML_EXTERN
        unsigned int getNumCurveSegments() const;


        /**
         * Creates a new LineSegment and adds it to the end
         * of the list.
         * A reference to the new LineSegment object is returned.
         */
        LIBSBML_EXTERN
        LineSegment& createLineSegment();

        /**
         * Creates a new CubicBezier and adds it to the end
         * of the list.
         * A reference to the new CubicBezier object is returned.
         */
        LIBSBML_EXTERN
        CubicBezier& createCubicBezier();
         
};


#endif
