/**
 * Filename    : Layout.h
 * Description : SBML Layout Layout C Header
 * Organization: European Media Laboratories Research gGmbH
 * Created     : 2004-04-26
 *
 * Copyright 2004 European Media Laboratories Research gGmbH
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
 * documentation provided hereunder is on an "as is" basis, and the
 * European Media Laboratories Research gGmbH have no obligations to provide maintenance, support,
 * updates, enhancements or modifications.  In no event shall the
 * European Media Laboratories Research gGmbH be liable to any party for direct, indirect, special,
 * incidental or consequential damages, including lost profits, arising
 * out of the use of this software and its documentation, even if the
 * European Media Laboratories Research gGmbH have been advised of the possibility of such damage.  See
 * the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Bioinformatics Group
 *     European Media Laboratories Research gGmbH
 *     Schloss-Wolfsbrunnenweg 31c
 *     69118 Heidelberg
 *     Germany
 *
 *     http://www.eml-research.de/english/Research/BCB/
 *     mailto:ralph.gauges@eml-r.villa-bosch.de
 *
 * Contributor(s):
 */






#ifndef Layout_H__
#define Layout_H__


#include "sbml/ListOf.h"
#include "sbml/layout/Dimensions.h"
#include "sbml/layout/CompartmentGlyph.h"
#include "sbml/layout/SpeciesGlyph.h"
#include "sbml/layout/ReactionGlyph.h"
#include "sbml/layout/SpeciesReferenceGlyph.h"
#include "sbml/layout/GraphicalObject.h"
#include "sbml/layout/TextGlyph.h"

BEGIN_C_DECLS

typedef void Layout_t;


/**
 * Creates a new Layout and returns a pointer to it.
 */
LIBSBML_EXTERN
Layout_t*
Layout_create();

/**
 * Creates a new Layout with the given id and returns a pointer to it.
 */
LIBSBML_EXTERN
Layout_t*
Layout_createWith(const char* sid);

/**
 * Creates a Layout object from a template.
 */
LIBSBML_EXTERN
Layout_t*
Layout_createFrom(const Layout_t* temp);

/**
 * Creates a new Layout with the given width, height and depth and returns a pointer to it.
 * The depth value defaults to 0.0.
 */
LIBSBML_EXTERN
Layout_t*
Layout_createWithSize(const char* id,double width,double height,double depth);

/**
 * Creates a new Layout with the given Dimensions and returns a pointer to it.
 */
LIBSBML_EXTERN
Layout_t*
Layout_createWithDimensions(const char* id,const Dimensions_t* dimensions);

/** 
 * Frees the memory for the given layout.
 */
LIBSBML_EXTERN
void 
Layout_free(Layout_t* l);

/**
 * Sets the id of the layout.
 */
LIBSBML_EXTERN
void
Layout_setId(Layout_t* l,const char* sid);

/**
 * Adds a new compartment glyph to the list of compartment glyphs.
 */
LIBSBML_EXTERN
void
Layout_addCompartmentGlyph(Layout_t* l,CompartmentGlyph_t* cg);

/**
 * Adds a new species glyph to the list of species glyphs.
 */
LIBSBML_EXTERN
void
Layout_addSpeciesGlyph(Layout_t* l,SpeciesGlyph_t* sg);

/**
 * Adds a new reaction glyph to the list of reaction glyphs.
 */
LIBSBML_EXTERN
void
Layout_addReactionGlyph(Layout_t* l,ReactionGlyph_t* rg);

/**
 * Adds a new GraphicalObject to the list of additional graphical objects.
 */
LIBSBML_EXTERN
void
Layout_addAdditionalGraphicalObject(Layout_t* l,GraphicalObject_t* go);

/**
 * Adds a new TextGlyph to the list of text glyphs.
 */
LIBSBML_EXTERN
void
Layout_addTextGlyph(Layout_t* l,TextGlyph_t* go);


/**
 * Returns a pointer to the CompartmentGlyph with the given index.
 */
LIBSBML_EXTERN
CompartmentGlyph_t*
Layout_getCompartmentGlyph(const Layout_t* l,unsigned int index);

/**
 * Returns a pointer to the SpeciesGlyph with the given index.
 */
LIBSBML_EXTERN
SpeciesGlyph_t*
Layout_getSpeciesGlyph(const Layout_t* l,unsigned int index);


/**
 * Returns a pointer to the ReactionGlyph with the given index.
 */
LIBSBML_EXTERN
ReactionGlyph_t*
Layout_getReactionGlyph(const Layout_t* l,unsigned int index);


/**
 * Returns a pointer to the AdditionalGraphicalObject with the given index.
 */
LIBSBML_EXTERN
GraphicalObject_t*
Layout_getAdditionalGraphicalObject(const Layout_t* l,unsigned int index);

/**
 * Returns a pointer to the GraphicalObject with the given index.
 */
LIBSBML_EXTERN
TextGlyph_t*
Layout_getTextGlyph(const Layout_t* l,unsigned int index);


/**
 * Returns a pointer to the list of CompartmentGlyphs.
 */
LIBSBML_EXTERN
ListOf_t*
Layout_getListOfCompartmentGlyphs(Layout_t* l);

/**
 * Returns a pointer to the list of SpeciesGlyphs.
 */
LIBSBML_EXTERN
ListOf_t*
Layout_getListOfSpeciesGlyphs(Layout_t* l);


/**
 * Returns a pointer to the list of ReactionGlyphs.
 */
LIBSBML_EXTERN
ListOf_t*
Layout_getListOfReactionGlyphs(Layout_t* l);


/**
 * Returns a pointer to the list of additional GraphicalObjects.
 */
LIBSBML_EXTERN
ListOf_t*
Layout_getListOfAdditionalGraphicalObjects(Layout_t* l);

/**
 * Returns a pointer to the list of TextGlyphs.
 */
LIBSBML_EXTERN
ListOf_t*
Layout_getListOfTextGlyphs(Layout_t* l);



/**
 * Returns a the id of the layout.
 */
LIBSBML_EXTERN
const char*
Layout_getId(const Layout_t* l);

/**
 * Returns the number of CompartmentGlyphs.
 */
LIBSBML_EXTERN
unsigned int
Layout_getNumCompartmentGlyphs(const Layout_t* l);

/**
 * Returns the number of SpeciesGlyphs.
 */
LIBSBML_EXTERN
unsigned int
Layout_getNumSpeciesGlyphs(const Layout_t* l);


/**
 * Returns the number of ReactionGlyphs.
 */
LIBSBML_EXTERN
unsigned int
Layout_getNumReactionGlyphs(const Layout_t* l);


/**
 * Returns the number of additional GraphicalObjects.
 */
LIBSBML_EXTERN
unsigned int
Layout_getNumAdditionalGraphicalObjects(const Layout_t* l);

/**
 * Returns the number of TextGlyphs.
 */
LIBSBML_EXTERN
unsigned int
Layout_getNumTextGlyphs(const Layout_t* l);



/**
 * Returns 0 if the id has not been set, 1 otherwise
 */
LIBSBML_EXTERN
int
Layout_isSetId(const Layout_t* l);

/**
 * Removes the compartment glyph with the given index.
 * If the index is invalid, nothing is deleted.
 */ 
LIBSBML_EXTERN
CompartmentGlyph_t*
Layout_removeCompartmentGlyph(Layout_t* l,unsigned int index);

/**
 * Removes the species glyph with the given index.
 * If the index is invalid, nothing is deleted.
 */ 
LIBSBML_EXTERN
SpeciesGlyph_t*
Layout_removeSpeciesGlyph(Layout_t* l,unsigned int index);

/**
 * Removes the reaction glyph with the given index.
 * If the index is invalid, nothing is deleted.
 */ 
LIBSBML_EXTERN
ReactionGlyph_t*
Layout_removeReactionGlyph(Layout_t* l,unsigned int index);
 
/**
 * Removes the text glyph with the given index.
 * If the index is invalid, nothing is deleted.
 */ 
LIBSBML_EXTERN
TextGlyph_t*
Layout_removeTextGlyph(Layout_t* l,unsigned int index);
 
/**
 * Removes the graphical object with the given index.
 * If the index is invalid, nothing is deleted.
 */ 
LIBSBML_EXTERN
GraphicalObject_t*
Layout_removeAdditionalGraphicalObject(Layout_t* l,unsigned int index);
    
/**
 * Does nothing since no defaults are defined for Layout.
 */ 
LIBSBML_EXTERN
void
Layout_initDefaults(Layout_t* l);


/**
 * Creates a ComparmentGlyph_t object, adds it to the end
 * of the compartment glyphs objects list and returns a
 * pointer to the newly created object.
 */
LIBSBML_EXTERN
CompartmentGlyph_t*
Layout_createCompartmentGlyph(Layout_t*);

/**
 * Creates a SpeciesGlyph object, adds it to the end
 * of the species glyphs objects list and returns a
 * pointer to the newly created object.
 */
LIBSBML_EXTERN
SpeciesGlyph_t*
Layout_createSpeciesGlyph(Layout_t*);


/**
 * Creates a ReactionGlyph_t object, adds it to the end
 * of the reaction glyphs objects list and returns a
 * pointer to the newly created object.
 */
LIBSBML_EXTERN
ReactionGlyph_t*
Layout_createReactionGlyph(Layout_t*);


/**
 * Creates a TextGlyph_t object, adds it to the end
 * of the text glyphs objects list and returns a
 * pointer to the newly created object.
 */
LIBSBML_EXTERN
TextGlyph_t*
Layout_createTextGlyph(Layout_t*);


/**
 * Creates a GraphicalObject object, adds it to the end
 * of the additional graphical objects list and returns a
 * pointer to the newly created object.
 */
LIBSBML_EXTERN
GraphicalObject_t*
Layout_createAdditionalGraphicalObject(Layout_t*);




END_C_DECLS

#endif
