/**
 * Filename    : Layout.hpp
 * Description : SBML Layout Layout C++ Header
 * Organization: European Media Laboratories Research gGmbH
 * Created     : 2004-07-15
 *
 * Copyright 2004 European Media Laboratories Research gGmbH
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
 * documentation provided hereunder is on an "as is" basis, and the
 * European Media Laboratories Research gGmbH have no obligations to provide maintenance, support,
 * updates, enhancements or modifications.  In no event shall the
 * European Media Laboratories Research gGmbH be liable to any party for direct, indirect, special,
 * incidental or consequential damages, including lost profits, arising
 * out of the use of this software and its documentation, even if the
 * European Media Laboratories Research gGmbH have been advised of the possibility of such damage.  See
 * the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Bioinformatics Group
 *     European Media Laboratories Research gGmbH
 *     Schloss-Wolfsbrunnenweg 31c
 *     69118 Heidelberg
 *     Germany
 *
 *     http://www.eml-research.de/english/Research/BCB/
 *     mailto:ralph.gauges@eml-r.villa-bosch.de
 *
 * Contributor(s):
 */

#ifndef Layout_HPP__
#define Layout_HPP__

#include <string>
#include "sbml/SBase.hpp"

#include "sbml/layout/Dimensions.hpp"
#include "sbml/ListOf.hpp"
#include "sbml/layout/CompartmentGlyph.hpp"
#include "sbml/layout/SpeciesGlyph.hpp"
#include "sbml/layout/ReactionGlyph.hpp"
#include "sbml/layout/TextGlyph.hpp"
#include "sbml/layout/GraphicalObject.hpp"

class Layout:public SBase{

    protected:
        std::string id;
        Dimensions dimensions;
        ListOf compartmentGlyphs;
        ListOf speciesGlyphs;
        ListOf reactionGlyphs;
        ListOf textGlyphs;
        ListOf additionalGraphicalObjects;

        GraphicalObject* removeObjectWithId(ListOf& list,const std::string& id);

        GraphicalObject* getObjectWithId(const ListOf& list,const std::string id) const;


    public:

        /**
         * Creates a new Layout.
         */
        LIBSBML_EXTERN
        Layout();


        /**
         * Creates a new Layout with the given id and dimensions.
         */
        LIBSBML_EXTERN
        Layout(const std::string& id,const Dimensions& dimensions);

        /**
         * Destructor.
         */ 
        LIBSBML_EXTERN
        virtual ~Layout();

        /**
         * Does nothing since no defaults are defined for Layout.
         */ 
        LIBSBML_EXTERN
        void initDefaults();    
        
        /**
         * Returns the id.
         */ 
        LIBSBML_EXTERN        
        const std::string getId() const;
        

        /**
         * Sets the id to a copy of the given string.
         */ 
        LIBSBML_EXTERN  
        void setId(const std::string& id);


        /**
         * Returns the dimensions of the layout.
         */ 
        LIBSBML_EXTERN
        const Dimensions& getDimensions() const;

        /**
         * Returns the dimensions of the layout.
         */ 
        LIBSBML_EXTERN
        Dimensions& getDimensions();

        /**
         * Sets the dimensions of the layout.
         */ 
        LIBSBML_EXTERN  
        void setDimensions(const Dimensions& dimensions);

        /**
         * Returns the ListOf object that holds all compartment glyphs.
         */ 
        LIBSBML_EXTERN
        const ListOf& getListOfCompartmentGlyphs() const;


        /**
         * Returns the ListOf object that holds all species glyphs.
         */ 
        LIBSBML_EXTERN 
        const ListOf& getListOfSpeciesGlyphs() const;
        

        /**
         * Returns the ListOf object that holds all reaction glyphs.
         */ 
        LIBSBML_EXTERN 
        const ListOf& getListOfReactionGlyphs() const;
        

        /**
         * Returns the ListOf object that holds all text glyphs.
         */ 
        LIBSBML_EXTERN 
        const ListOf& getListOfTextGlyphs() const;
        

        /**
         * Returns the ListOf object that holds all additonal graphical objects.
         */ 
        LIBSBML_EXTERN 
        const ListOf& getListOfAdditionalGraphicalObjects() const;


        /**
         * Returns the ListOf object that holds all compartment glyphs.
         */ 
        LIBSBML_EXTERN
        ListOf& getListOfCompartmentGlyphs();


        /**
         * Returns the ListOf object that holds all species glyphs.
         */ 
        LIBSBML_EXTERN 
        ListOf& getListOfSpeciesGlyphs();
        

        /**
         * Returns the ListOf object that holds all reaction glyphs.
         */ 
        LIBSBML_EXTERN 
        ListOf& getListOfReactionGlyphs();
        

        /**
         * Returns the ListOf object that holds all text glyphs.
         */ 
        LIBSBML_EXTERN 
        ListOf& getListOfTextGlyphs();
        

        /**
         * Returns the ListOf object that holds all additional graphical objects.
         */ 
        LIBSBML_EXTERN 
        ListOf& getListOfAdditionalGraphicalObjects();


        /**
         * Returns the compartment glyph with the given index.
         * If the index is invalid, NULL is returned.
         */ 
        LIBSBML_EXTERN 
        CompartmentGlyph* getCompartmentGlyph(unsigned int index) const;


        /**
         * Returns the species glyph with the given index.
         * If the index is invalid, NULL is returned.
         */ 
        LIBSBML_EXTERN
        SpeciesGlyph* getSpeciesGlyph(unsigned int index) const;
        

        /**
         * Returns the reaction glyph with the given index.
         * If the index is invalid, NULL is returned.
         */ 
        LIBSBML_EXTERN
        ReactionGlyph* getReactionGlyph(unsigned int index) const;
        

        /**
         * Returns the text glyph with the given index.
         * If the index is invalid, NULL is returned.
         */ 
        LIBSBML_EXTERN
        TextGlyph* getTextGlyph(unsigned int index) const;
       

        /**
         * Returns the additional graphical object with the given index.
         * If the index is invalid, NULL is returned.
         */ 
        LIBSBML_EXTERN 
        GraphicalObject* getAdditionalGraphicalObject(unsigned int index) const;


        /**
         * Returns the compartment glyph that has the given id, or NULL if no compartment glyph has the id.
         */
        LIBSBML_EXTERN
        CompartmentGlyph* getCompartmentGlyph(const std::string& id) const;


        /**
         * Returns the species glyph that has the given id, or NULL if no species glyph has the id.
         */
        LIBSBML_EXTERN
        SpeciesGlyph* getSpeciesGlyph(const std::string& id) const;
        

        /**
         * Returns the reaction glyph that has the given id, or NULL if no reaction glyph has the id.
         */
        LIBSBML_EXTERN
        ReactionGlyph* getReactionGlyph(const std::string& id) const;
        

        /**
         * Returns the text glyph that has the given id, or NULL if no text glyph has the id.
         */
        LIBSBML_EXTERN
        TextGlyph* getTextGlyph(const std::string& id) const;
       

        /**
         * Returns the additional graphical object that has the given id, or NULL if no graphical object has the id.
         */
        LIBSBML_EXTERN
        GraphicalObject* getAdditionalGraphicalObject(const std::string& id) const;


        /**
         * Adds a new compartment glyph.
         */
        LIBSBML_EXTERN
        void addCompartmentGlyph(CompartmentGlyph& glyph);
        

        /**
         * Adds a new species glyph.
         */
        LIBSBML_EXTERN
        void addSpeciesGlyph(SpeciesGlyph& glyph);
        

        /**
         * Adds a new reaction glyph.
         */
        LIBSBML_EXTERN
        void addReactionGlyph(ReactionGlyph& glyph);
        

        /**
         * Adds a new text glyph.
         */
        LIBSBML_EXTERN
        void addTextGlyph(TextGlyph& glyph);
        

        /**
         * Adds a new additional graphical object glyph.
         */
        LIBSBML_EXTERN
        void addAdditionalGraphicalObject(GraphicalObject& glyph);
       
        /**
         * Returns the number of compartment glyphs for the layout.
         */
        LIBSBML_EXTERN
        unsigned int getNumCompartmentGlyphs() const;
        

        /**
         * Returns the number of species glyphs for the layout.
         */
        LIBSBML_EXTERN 
        unsigned int getNumSpeciesGlyphs() const;
        

        /**
         * Returns the number of reaction glyphs for the layout.
         */
        LIBSBML_EXTERN
        unsigned int getNumReactionGlyphs() const;
        

        /**
         * Returns the number of text glyphs for the layout.
         */
        LIBSBML_EXTERN 
        unsigned int getNumTextGlyphs() const;
        

        /**
         * Returns the number of additional graphical objects for the layout.
         */
        LIBSBML_EXTERN
        unsigned int getNumAdditionalGraphicalObjects() const;

        /**
         * Returns true if the id is not the empty string.
         */ 
        LIBSBML_EXTERN
        bool isSetId() const;    

        /**
         * Creates a CompartmentGlyph object, adds it to the end
         * of the compartment glyph objects list and returns a
         * reference to the newly created object.
         */
        LIBSBML_EXTERN
        CompartmentGlyph& createCompartmentGlyph();

        /**
         * Creates a SpeciesGlyph object, adds it to the end
         * of the species glyph objects list and returns a
         * reference to the newly created object.
         */
        LIBSBML_EXTERN
        SpeciesGlyph& createSpeciesGlyph();
        
        /**
         * Creates a ReactionGlyph object, adds it to the end
         * of the reaction glyph objects list and returns a
         * reference to the newly created object.
         */
        LIBSBML_EXTERN
        ReactionGlyph& createReactionGlyph();
        
        /**
         * Creates a TextGlyph object, adds it to the end
         * of the text glyph objects list and returns a
         * reference to the newly created object.
         */
        LIBSBML_EXTERN
        TextGlyph& createTextGlyph();
        
        /**
         * Creates a GraphicalObject object, adds it to the end
         * of the additional graphical objects list and returns a
         * reference to the newly created object.
         */
        LIBSBML_EXTERN
        GraphicalObject& createAdditionalGraphicalObject();

        /**
         * Creates a new SpeciesReferenceGlyph for the last ReactionGlyph
         * and adds it to its list of SpeciesReferenceGlyph objects.
         * A reference to the newly created object is returned.
         */
        LIBSBML_EXTERN
        SpeciesReferenceGlyph* createSpeciesReferenceGlyph();        

        /**
         * Creates a new LineSegment for the Curve object of the last ReactionGlyph
         * or the last SpeciesReferenceGlyph in the last ReactionGlyph 
         * and adds it to its list of SpeciesReferenceGlyph objects.
         * A reference to the newly created object is returned.
         */
        LIBSBML_EXTERN
        LineSegment* createLineSegment();        

        /**
         * Creates a new CubicBezier for the Curve object of the last ReactionGlyph
         * or the last SpeciesReferenceGlyph in the last ReactionGlyph 
         * and adds it to its list of SpeciesReferenceGlyph objects.
         * A reference to the newly created object is returned.
         */
        LIBSBML_EXTERN
        CubicBezier* createCubicBezier();        

};


#endif
