/**
 * Filename    : SpeciesReferenceGlyph.hpp
 * Description : SBML Layout SpeciesReferenceGlyph C++ Header
 * Organization: European Media Laboratories Research gGmbH
 * Created     : 2004-07-15
 *
 * Copyright 2004 European Media Laboratories Research gGmbH
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
 * documentation provided hereunder is on an "as is" basis, and the
 * European Media Laboratories Research gGmbH have no obligations to provide maintenance, support,
 * updates, enhancements or modifications.  In no event shall the
 * European Media Laboratories Research gGmbH be liable to any party for direct, indirect, special,
 * incidental or consequential damages, including lost profits, arising
 * out of the use of this software and its documentation, even if the
 * European Media Laboratories Research gGmbH have been advised of the possibility of such damage.  See
 * the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Bioinformatics Group
 *     European Media Laboratories Research gGmbH
 *     Schloss-Wolfsbrunnenweg 31c
 *     69118 Heidelberg
 *     Germany
 *
 *     http://www.eml-research.de/english/Research/BCB/
 *     mailto:ralph.gauges@eml-r.villa-bosch.de
 *
 * Contributor(s):
 */

#ifndef SpeciesReferenceGlyph_HPP__
#define SpeciesReferenceGlyph_HPP__

#include <string>
#include "sbml/layout/GraphicalObject.hpp"
#include "sbml/layout/SpeciesReferenceRole.h"
#include "sbml/layout/Curve.hpp"

typedef SpeciesReferenceRole_t SpeciesReferenceRole;


class SpeciesReferenceGlyph:public GraphicalObject{
    protected:
        std::string speciesReference;
        std::string speciesGlyph;
        SpeciesReferenceRole role;
        Curve* curve;

    public:
        /**
         * Creates a new SpeciesReferenceGlyph.
         * The id if the associated species reference and the id of the associated species glyph are set to the empty string.
         * The role is set to SPECIES_ROLE_UNDEFINED.
         */
        LIBSBML_EXTERN
        SpeciesReferenceGlyph();
        
        /**
         * Creates a new SpeciesReferenceGlyph.
         * The id is given as the first argument, the id of the associated species reference is given as the second argument.
         * The third argument is the id of the associated species glpyh and
         * the fourth argument is the role.
         */ 
        LIBSBML_EXTERN
        SpeciesReferenceGlyph(const std::string& sid,const std::string& speciesReferenceId,const std::string& speciesGlyphId,SpeciesReferenceRole role);
        
        /**
         * Destructor.
         */ 
        LIBSBML_EXTERN
        virtual ~SpeciesReferenceGlyph(); 
        
        /**
         * Returns the id of the associated SpeciesGlyph.
         */ 
        LIBSBML_EXTERN
        const std::string& getSpeciesGlyphId() const;
        
        /**
         * Sets the id of the associated species glyph.
         */ 
        LIBSBML_EXTERN
        void setSpeciesGlyphId(const std::string& speciesGlyphId); 
        
        /**
         * Returns the id of the associated species reference.
         */ 
        LIBSBML_EXTERN
        const std::string& getSpeciesReferenceId() const;
        
        /**
         * Sets the id of the associated species reference.
         */ 
        LIBSBML_EXTERN
        void setSpeciesReferenceId(const std::string& id); 
        
        /**
         * Returns the role.
         */ 
        LIBSBML_EXTERN
        SpeciesReferenceRole getRole() const;
        
        /**
         * Sets the role based on a string.
         * The String can be one of
         * SUBSTRATE
         * PRODUCT
         * SIDESUBSTRATE
         * SIDEPRODUCT
         * MODIFIER
         * ACTIVATOR
         * INHIBITOR    
         */ 
        LIBSBML_EXTERN
        void setRole(const std::string& role); 

        /**
         * Sets the role.
         */ 
        LIBSBML_EXTERN
        void setRole(SpeciesReferenceRole role); 
        
        /**
         * Returns the curve object for the species reference glyph
         */ 
        LIBSBML_EXTERN
        Curve* getCurve() const;

        /**
         * Sets the curve object for the species reference glyph.
         */ 
        LIBSBML_EXTERN
        void setCurve(Curve* curve);
       
        /**
         * Returns true if the curve consists of one or more segments.
         */ 
        LIBSBML_EXTERN
        bool isSetCurve() const;

        /**
         * Returns true if the id of the associated species glpyh is not the empty string.
         */ 
        LIBSBML_EXTERN
        bool isSetSpeciesGlyphId() const;
        
        /**
         * Returns true if the id of the associated species reference is not the empty string.
         */ 
        LIBSBML_EXTERN
        bool isSetSpeciesReferenceId() const;
        
        /**
         * Returns true of role is different from SPECIES_ROLE_UNDEFINED.
         */ 
        LIBSBML_EXTERN
        bool isSetRole() const;
        
        /**
         * Calls initDefaults on GraphicalObject and sets role to SPECIES_ROLE_UNDEFINED.
         */ 
        LIBSBML_EXTERN
        void initDefaults();

        /**
         * Creates a new LineSegment object, adds it to the end of the list of curve
         * segment objects of the curve and returns a reference to the newly created object.
         */
        LIBSBML_EXTERN
        LineSegment&
        SpeciesReferenceGlyph::createLineSegment();

        /**
         * Creates a new CubicBezier object, adds it to the end of the list of curve
         * segment objects of the curve and returns a reference to the newly created object.
         */
        LIBSBML_EXTERN
        CubicBezier&
        SpeciesReferenceGlyph::createCubicBezier();

};


#endif
