/**
 * Filename    : SimpleSpeciesReference.cpp
 * Description : SBML SimpleSpeciesReference
 * Author(s)   : SBML Development Group <sbml-team@caltech.edu>
 * Organization: JST ERATO Kitano Symbiotic Systems Project
 * Created     : 2003-04-29
 * Revision    : $Id: SimpleSpeciesReference.cpp,v 1.2 2004/07/24 01:06:02 mhucka Exp $
 * Source      : $Source: /cvsroot/sbml/libsbml/src/SimpleSpeciesReference.cpp,v $
 *
 * Copyright 2003 California Institute of Technology and
 * Japan Science and Technology Corporation.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
 * documentation provided hereunder is on an "as is" basis, and the
 * California Institute of Technology and Japan Science and Technology
 * Corporation have no obligations to provide maintenance, support,
 * updates, enhancements or modifications.  In no event shall the
 * California Institute of Technology or the Japan Science and Technology
 * Corporation be liable to any party for direct, indirect, special,
 * incidental or consequential damages, including lost profits, arising
 * out of the use of this software and its documentation, even if the
 * California Institute of Technology and/or Japan Science and Technology
 * Corporation have been advised of the possibility of such damage.  See
 * the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ben Bornstein
 *     The Systems Biology Markup Language Development Group
 *     ERATO Kitano Symbiotic Systems Project
 *     Control and Dynamical Systems, MC 107-81
 *     California Institute of Technology
 *     Pasadena, CA, 91125, USA
 *
 *     http://www.cds.caltech.edu/erato
 *     mailto:sbml-team@caltech.edu
 *
 * Contributor(s):
 */


#include "sbml/util.h"
#include "sbml/SimpleSpeciesReference.h"
#include "sbml/SimpleSpeciesReference.hpp"


/**
 * Creates a new SimpleSpeciesReference, optionally with its species
 * attribute set.
 */
LIBSBML_EXTERN
SimpleSpeciesReference::SimpleSpeciesReference (const std::string& s) :
  SBase(), species(s)
#ifdef MAKE_LAYOUT
  , id("")
#endif /* MAKE_LAYOUT */  
{
}


/**
 * @return the species for this SimpleSpeciesReference.
 */
LIBSBML_EXTERN
const std::string&
SimpleSpeciesReference::getSpecies () const
{
  return species;
}


/**
 * @return true if the species for this SimpleSpeciesReference has been
 * set, false otherwise.
 */
LIBSBML_EXTERN
bool
SimpleSpeciesReference::isSetSpecies () const
{
  return ! species.empty();
}


/**
 * Sets the species of this SimpleSpeciesReference to a copy of sid.
 */
LIBSBML_EXTERN
void
SimpleSpeciesReference::setSpecies (const std::string& sid)
{
  species = sid;
}

#ifdef MAKE_LAYOUT  
/**
 * Sets the id for the species reference.
 */
LIBSBML_EXTERN
void SimpleSpeciesReference::setId(const std::string& id){
    this->id=id;
}

/**
 * Returns the id for the species reference.
 */
LIBSBML_EXTERN
const std::string& SimpleSpeciesReference::getId() const{
    return this->id;
}

/**
 * Returns true if the id of the species reference is not equal to the empty string. 
 */
LIBSBML_EXTERN
bool SimpleSpeciesReference::isSetId() const{
    return !(this->id=="");
}

/**
 * Unsets the id for the species reference.
 */
LIBSBML_EXTERN
void SimpleSpeciesReference::unsetId(){
    this->id="";
}

#endif /* MAKE_LAYOUT */
 


/**
 * @return the species for this SimpleSpeciesReference.
 */
LIBSBML_EXTERN
const char *
SimpleSpeciesReference_getSpecies (const SimpleSpeciesReference_t *ssr)
{
  const SimpleSpeciesReference* x =
    static_cast<const SimpleSpeciesReference*>(ssr);


  return x->isSetSpecies() ? x->getSpecies().c_str() : NULL;
}


/**
 * @return 1 if the species for this SimpleSpeciesReference has been set, 0
 * otherwise.
 */
LIBSBML_EXTERN
int
SimpleSpeciesReference_isSetSpecies (const SimpleSpeciesReference_t *ssr)
{
  return (int) static_cast<const SimpleSpeciesReference*>(ssr)->isSetSpecies();
}


/**
 * Sets the species of this SimpleSpeciesReference to a copy of sid.
 */
LIBSBML_EXTERN
void
SimpleSpeciesReference_setSpecies ( SimpleSpeciesReference_t *ssr,
                                    const char *sid )
{
  static_cast<SimpleSpeciesReference*>(ssr)->setSpecies(sid ? sid : "");
}

#ifdef MAKE_LAYOUT
/**
 * Returns the id for the species reference.
 */ 
LIBSBML_EXTERN
const char*
SimpleSpeciesReference_getId(const SimpleSpeciesReference_t* sr){
    return static_cast<const SimpleSpeciesReference*>(sr)->getId().c_str();
}

/**
 * Sets the id for the given SpeciesReference.
 */ 
LIBSBML_EXTERN
void
SimpleSpeciesReference_setId(SimpleSpeciesReference_t* sr,const char* id){
    static_cast<SimpleSpeciesReference*>(sr)->setId(id);
}


/**
 * Returns 0 if the id is not set.
 */ 
LIBSBML_EXTERN
int
SimpleSpeciesReference_isSetId(const SimpleSpeciesReference_t* sr){
    return (int)static_cast<const SimpleSpeciesReference*>(sr)->isSetId();
}

/**
 * Sets the id to the empty string.
 */ 
LIBSBML_EXTERN
void
SimpleSpeciesReference_unsetId(SimpleSpeciesReference_t* sr){
    static_cast<SimpleSpeciesReference*>(sr)->unsetId();
}

#endif /* MAKE_LAYOUT */


/**
 * The SimpleSpeciesReferenceCmp function compares the string sid to
 * ssr->species.
 *
 * @returns an integer less than, equal to, or greater than zero if sid is
 * found to be, respectively, less than, to match or be greater than
 * ssr->species.  Returns -1 if either sid or ssr->species is NULL.
 */
LIBSBML_EXTERN
int
SimpleSpeciesReferenceCmp(const char *sid, const SimpleSpeciesReference_t *ssr)
{
  int result = -1;


  if (sid != NULL && SimpleSpeciesReference_isSetSpecies(ssr))
  {
    result = strcmp(sid, SimpleSpeciesReference_getSpecies(ssr));
  }

  return result;
}
