/**
 * Filename    : TestSpeciesReference.c
 * Description : SpeciesReference unit tests
 * Author(s)   : SBML Development Group <sbml-team@caltech.edu>
 * Organization: JST ERATO Kitano Symbiotic Systems Project
 * Created     : 2002-11-25
 * Revision    : $Id: TestSpeciesReference.c,v 1.11 2004/07/24 01:06:03 mhucka Exp $
 * Source      : $Source: /cvsroot/sbml/libsbml/src/TestSpeciesReference.c,v $
 *
 * Copyright 2002 California Institute of Technology and
 * Japan Science and Technology Corporation.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
 * documentation provided hereunder is on an "as is" basis, and the
 * California Institute of Technology and Japan Science and Technology
 * Corporation have no obligations to provide maintenance, support,
 * updates, enhancements or modifications.  In no event shall the
 * California Institute of Technology or the Japan Science and Technology
 * Corporation be liable to any party for direct, indirect, special,
 * incidental or consequential damages, including lost profits, arising
 * out of the use of this software and its documentation, even if the
 * California Institute of Technology and/or Japan Science and Technology
 * Corporation have been advised of the possibility of such damage.  See
 * the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ben Bornstein
 *     The Systems Biology Markup Language Development Group
 *     ERATO Kitano Symbiotic Systems Project
 *     Control and Dynamical Systems, MC 107-81
 *     California Institute of Technology
 *     Pasadena, CA, 91125, USA
 *
 *     http://www.cds.caltech.edu/erato
 *     mailto:sbml-team@caltech.edu
 *
 * Contributor(s):
 */


#include <check.h>

#include "sbml/common.h"
#include "sbml/MathMLReader.h"
#include "sbml/SpeciesReference.h"


static SpeciesReference_t *SR;


void
SpeciesReferenceTest_setup (void)
{
  SR = SpeciesReference_create();

  if (SR == NULL)
  {
    fail("SpeciesReference_create() returned a NULL pointer.");
  }
}


void
SpeciesReferenceTest_teardown (void)
{
  SpeciesReference_free(SR);
}


START_TEST (test_SpeciesReference_create)
{
  fail_unless( SBase_getTypeCode  (SR) == SBML_SPECIES_REFERENCE, NULL );
  fail_unless( SBase_getMetaId    (SR) == NULL, NULL );
  fail_unless( SBase_getNotes     (SR) == NULL, NULL );
  fail_unless( SBase_getAnnotation(SR) == NULL, NULL );

  fail_unless( SpeciesReference_getSpecies          (SR) == NULL, NULL );
  fail_unless( SpeciesReference_getStoichiometry    (SR) == 1   , NULL );
  fail_unless( SpeciesReference_getStoichiometryMath(SR) == NULL, NULL );
  fail_unless( SpeciesReference_getDenominator      (SR) == 1   , NULL );

  fail_unless( !SpeciesReference_isSetSpecies(SR), NULL );
  fail_unless( !SpeciesReference_isSetStoichiometryMath(SR), NULL );
}
END_TEST


START_TEST (test_SpeciesReference_createWith)
{
  SpeciesReference_t *sr = SpeciesReference_createWith("s3", 4, 2);


  fail_unless( SBase_getTypeCode  (sr) == SBML_SPECIES_REFERENCE, NULL );
  fail_unless( SBase_getMetaId    (sr) == NULL, NULL );
  fail_unless( SBase_getNotes     (sr) == NULL, NULL );
  fail_unless( SBase_getAnnotation(sr) == NULL, NULL );

  fail_unless( !strcmp(SpeciesReference_getSpecies(sr), "s3"), NULL );

  fail_unless( SpeciesReference_getStoichiometry(sr) == 4   , NULL );
  fail_unless( SpeciesReference_getDenominator  (sr) == 2   , NULL );

  fail_unless( SpeciesReference_isSetSpecies(sr), NULL );

  SpeciesReference_free(sr);
}
END_TEST


START_TEST (test_SpeciesReference_free_NULL)
{
  SpeciesReference_free(NULL);
}
END_TEST


START_TEST (test_SpeciesReference_setSpecies)
{
  char *species = "X0";


  SpeciesReference_setSpecies(SR, species);

  fail_unless( !strcmp(SpeciesReference_getSpecies(SR), species), NULL );
  fail_unless( SpeciesReference_isSetSpecies(SR), NULL );

  if (SpeciesReference_getSpecies(SR) == species)
  {
    fail("SpeciesReference_setSpecies(...) did not make a copy of string.");
  }

  /* Reflexive case (pathological) */
  SpeciesReference_setSpecies(SR, SpeciesReference_getSpecies(SR));
  fail_unless( !strcmp(SpeciesReference_getSpecies(SR), species), NULL );

  SpeciesReference_setSpecies(SR, NULL);
  fail_unless( !SpeciesReference_isSetSpecies(SR), NULL );

  if (SpeciesReference_getSpecies(SR) != NULL)
  {
    fail("SpeciesReference_setSpecies(SR, NULL) did not clear string.");
  }
}
END_TEST


START_TEST (test_SpeciesReference_setStoichiometryMath)
{
  const ASTNode_t  *n;
  const ASTNode_t  *math;
  MathMLDocument_t *d;


  d    = readMathMLFromString("<cn type='rational'>1<sep/>2</cn>");
  math = MathMLDocument_getMath(d);

  SpeciesReference_setStoichiometryMath(SR, (ASTNode_t*) math);

  fail_unless( SpeciesReference_getStoichiometryMath(SR) == math, NULL );
  fail_unless( SpeciesReference_isSetStoichiometryMath(SR), NULL );

  /* Reflexive case (pathological) */
  n = SpeciesReference_getStoichiometryMath(SR);
  SpeciesReference_setStoichiometryMath(SR, (ASTNode_t *) n);

  n = SpeciesReference_getStoichiometryMath(SR);
  fail_unless( n == math, NULL );

  SpeciesReference_setStoichiometryMath(SR, NULL);
  fail_unless( !SpeciesReference_isSetStoichiometryMath(SR), NULL );

  if (SpeciesReference_getStoichiometryMath(SR) != NULL)
  {
    fail( "SpeciesReference_setStoichiometryMath(SR, NULL) "
          "did not clear ASTNode." );
  }

  /* MathMLDocument_free(d); */
}
END_TEST


Suite *
create_suite_SpeciesReference (void)
{
  Suite *suite = suite_create("SpeciesReference");
  TCase *tcase = tcase_create("SpeciesReference");


  tcase_add_checked_fixture( tcase,
                             SpeciesReferenceTest_setup,
                             SpeciesReferenceTest_teardown );

  tcase_add_test( tcase, test_SpeciesReference_create               );
  tcase_add_test( tcase, test_SpeciesReference_createWith           );
  tcase_add_test( tcase, test_SpeciesReference_free_NULL            );
  tcase_add_test( tcase, test_SpeciesReference_setSpecies           );
  tcase_add_test( tcase, test_SpeciesReference_setStoichiometryMath );

  suite_add_tcase(suite, tcase);

  return suite;
}
